﻿using Hotline.Caching.Interfaces;
using Hotline.Orders;
using Hotline.Repository.SqlSugar.Snapshot;
using Hotline.Settings;
using Hotline.Share.Dtos.Snapshot;
using Hotline.Share.Enums.Snapshot;
using Hotline.Share.Enums.User;
using Hotline.Share.Tools;
using Hotline.Snapshot;
using Hotline.Snapshot.Interfaces;
using Hotline.ThirdAccountDomainServices;
using Hotline.ThirdAccountDomainServices.Interfaces;
using IdentityModel;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Claims;
using System.Text;
using System.Threading.Tasks;
using XF.Domain.Dependency;

namespace Hotline.Application.Snapshot;
public class SnapshotThirdAccountSupplier : IThirdAccountSupplier, IScopeDependency
{
    private readonly IThirdAccountRepository _thirdAccountRepository;
    private readonly IThirdAccountDomainService _thirdAccountDomainService;
    private readonly IVolunteerRepository _volunteerRepository;
    private readonly ISnapshotUserInfoRepository _snapshotUserInfoRepository;
    private readonly ISystemSettingCacheManager _systemSettingCacheManager;
    private readonly ISystemLogRepository _systemLog;

    public SnapshotThirdAccountSupplier(IThirdAccountRepository thirdAccountRepository, IVolunteerRepository volunteerRepository, ISnapshotUserInfoRepository snapshotUserInfoRepository, ISystemSettingCacheManager systemSettingCacheManager, ISystemLogRepository systemLog, IThirdAccountDomainService thirdAccountDomainService)
    {
        _thirdAccountRepository = thirdAccountRepository;
        _volunteerRepository = volunteerRepository;
        _snapshotUserInfoRepository = snapshotUserInfoRepository;
        _systemSettingCacheManager = systemSettingCacheManager;
        _systemLog = systemLog;
        _thirdAccountDomainService = thirdAccountDomainService;
    }

    public async Task<List<Claim>> GetClaimAsync(ThirdAccount account, List<Claim> claims, CancellationToken token)
    {
        var userInfo = await _snapshotUserInfoRepository.GetAsync(account.ExternalId);
        if (userInfo != null)
        {
            var subject = claims.Find(m => m.Type == JwtClaimTypes.Subject);
            if (subject != null)
                claims.Remove(subject);
            claims.Add(new(JwtClaimTypes.Subject, userInfo.Id));
        }
        return claims;
    }

    public async Task<Dictionary<string, object>> GetLoginOutDataAsync(ThirdAccount thirdAccount, Dictionary<string, object> dicOutData, CancellationToken cancel)
    {
        var isVolunteer = await _volunteerRepository.IsVolunteerAsync(thirdAccount.PhoneNumber);
        var user = await _snapshotUserInfoRepository.GetAsync(thirdAccount.ExternalId);
        if (user == null || thirdAccount.ExternalId.IsNullOrEmpty())
        {
            user = new SnapshotUserInfo
            {
                CitizenType = EReadPackUserType.Citizen,
                Name = thirdAccount.UserName,
                PhoneNumber = thirdAccount.PhoneNumber,
            };
            user.Id = await _snapshotUserInfoRepository.AddAsync(user);
            await _thirdAccountDomainService.UpdateExternalIdAsync(thirdAccount.Id, thirdAccount.ExternalId, user.Id, cancel);
        }
        dicOutData.Add("UserType", user.CitizenType);
        dicOutData.Add("IsVolunteer", isVolunteer);
        dicOutData.Add("InvitationCode", user.InvitationCode);
        return dicOutData;
    }

    public async Task<ThirdTokenDto> GetThirdParameterAsync(ThirdTokenDto thirdDto, CancellationToken token)
    {
        if (thirdDto.ThirdType == EThirdType.WeChat)
        {
            thirdDto.AppId = _systemSettingCacheManager.WxOpenAppId;
            thirdDto.Secret = _systemSettingCacheManager.WxOpenAppSecret;
        }
        return await Task.FromResult(thirdDto);
    }

    public async Task RegisterAsync(ThirdAccount thirdAccount, CancellationToken token)
    {
        var userInfo = await _snapshotUserInfoRepository.GetByPhoneNumberAsync(thirdAccount.PhoneNumber);
        if (userInfo != null)
        {
            userInfo.CitizenType = EReadPackUserType.Guider;
            await _snapshotUserInfoRepository.UpdateAsync(userInfo);
            await _thirdAccountDomainService.UpdateExternalIdAsync(thirdAccount.Id, thirdAccount.ExternalId, userInfo.Id, token);
        }
        else
        {
            userInfo = new SnapshotUserInfo
            {
                PhoneNumber = thirdAccount.PhoneNumber,
                CitizenType = EReadPackUserType.Citizen,
            };
            userInfo.Id = await _snapshotUserInfoRepository.AddAsync(userInfo);
            await _thirdAccountDomainService.UpdateExternalIdAsync(thirdAccount.Id, thirdAccount.ExternalId, userInfo.Id, token);
        }
    }
}

